/* general */
.hide, .yui-skin-sam .yui-editor-panel label.hide {
    display: none;
}
.form_control .dfv-errors {
  color: red;
  font-weight: bold;
  display: inline;
  vertical-align: top;
}

form {
    border: thin solid black;
    margin: 2em;
}

fieldset {
    display: table;
}

legend, h2 {
    width: 100%;
    font-weight: bold;
    text-align: center;
}

.form_control {
    width: 100%;
    display: table-row;
}

.form_control .label {
    width: 30%;
    display: table-cell;
    vertical-align: top;
    padding-left: 5em;
}

.form_control input,
.form_control textarea {
    width: 70%;
    display: table-cell;
}

input.form_control[type="submit"] {
    width: 200px;
    margin: 1em;
}


.yui-toolbar-group-border,
.yui-toolbar-group-padding,
.yui-toolbar-group-height-width,
.yui-toolbar-group-textflow {
	display: none;
}
